/*
 * productionMode.cpp
 *
 *  Created on: Jan 24, 2018
 *      Author: wn
 */


#include "defines.h"

#define MQTT_MAX_PACKET_SIZE 256

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <PubSubClient.h>
#include "configuration.h"

#define BUF_SIZE 64


volatile uint32_t cnt = 0;
uint32_t uptime = 0;

WiFiClientSecure espClient;
PubSubClient client(espClient);


enum pressedState { IDLE, XPENDING, SHORT } ;

volatile pressedState pressed1 = IDLE;
volatile pressedState pressed2 = IDLE;
volatile pressedState pressed3 = IDLE;
volatile pressedState pressed4 = IDLE;


void change1() {
	static uint32_t lastEvent = 0;
	uint32_t currentEvent = micros();
	if (currentEvent > (lastEvent + configBlock.debounce)) {
		uint8_t buttonState = digitalRead(SWITCH_1);
		switch (pressed1) {
		case IDLE:
			digitalWrite(LED_1, HIGH);
			if (buttonState == LOW) {
				digitalWrite(LED_2, HIGH);
				pressed1 = XPENDING;
			}
			break;
		case XPENDING:
			digitalWrite(LED_1, LOW);
			if (buttonState == HIGH) {
				if (true || currentEvent > (lastEvent + (configBlock.shortPress * 1000))) {
					digitalWrite(LED_2, LOW);
					pressed1 = SHORT;
				}
			} else {
				pressed1 = IDLE;
			}
			break;
		default:
			pressed1 = IDLE;
		}
		lastEvent = currentEvent;
	}
}
void change2() {
	static uint32_t lastEvent = 0;
	uint32_t currentEvent = micros();
	if (currentEvent > (lastEvent + configBlock.debounce)) {
		uint8_t buttonState = digitalRead(SWITCH_2);
		switch (pressed2) {
		case IDLE:
			if (buttonState == LOW) {
				pressed2 = XPENDING;
			}
			break;
		case XPENDING:
			if (buttonState == HIGH) {
				if (currentEvent > (lastEvent + configBlock.shortPress)) {
					pressed2 = SHORT;
				}
			} else {
				pressed2 = IDLE;
			}
			break;
		default:
			pressed2 = IDLE;
		}
		lastEvent = currentEvent;
	}
}
void change3() {
	static uint32_t lastEvent = 0;
	uint32_t currentEvent = micros();
	if (currentEvent > (lastEvent + configBlock.debounce)) {
		uint8_t buttonState = digitalRead(SWITCH_3);
		switch (pressed3) {
		case IDLE:
			if (buttonState == LOW) {
				pressed3 = XPENDING;
			}
			break;
		case XPENDING:
			if (buttonState == HIGH) {
				if (currentEvent > (lastEvent + configBlock.shortPress)) {
					pressed3 = SHORT;
				}
			} else {
				pressed3 = IDLE;
			}
			break;
		default:
			pressed3 = IDLE;
		}
		lastEvent = currentEvent;
	}
}
void change4() {
	static uint32_t lastEvent = 0;
	uint32_t currentEvent = micros();
	if (currentEvent > (lastEvent + configBlock.debounce)) {
		uint8_t buttonState = digitalRead(SWITCH_4);
		switch (pressed4) {
		case IDLE:
			if (buttonState == LOW) {
				pressed4 = XPENDING;
			}
			break;
		case XPENDING:
			if (buttonState == HIGH) {
				if (currentEvent > (lastEvent + configBlock.shortPress)) {
					pressed4 = SHORT;
				}
			} else {
				pressed4 = IDLE;
			}
			break;
		default:
			pressed4 = IDLE;
		}
		lastEvent = currentEvent;
	}
}

void setup_wifi() {
	delay(10);
	WiFi.mode(WIFI_STA);

	// We start by connecting to a WiFi network
#ifdef DEBUG
	Serial.println();
	Serial.print("Connecting to ");
	Serial.println(configBlock.wifiSsid);
#endif

	WiFi.begin(configBlock.wifiSsid, configBlock.wifiKey);

	while (WiFi.status() != WL_CONNECTED) {
		delay(50);
#ifdef DEBUG
		Serial.print(".");
#endif
	}
	Serial.println("!");

#ifdef DEBUG
	Serial.println("");
	Serial.println("WiFi connected");
	Serial.println("IP address: ");
	Serial.println(WiFi.localIP());
#endif
}





void reconnect() {
	// Loop until we're reconnected
	while (!client.connected()) {
#ifdef DEBUG
		Serial.print("Attempting MQTT connection...");
#endif
		// Attempt to connect
		//char clientId[128];
		//snprintf(clientId, 127, "esp%s", WiFi.macAddress().c_str());
		if (client.connect(configBlock.mqttClientId, configBlock.mqttUser, configBlock.mqttPass)) {
#ifdef DEBUG
			Serial.println("connected");
#endif
//			client.setCallback(callback);

			// Once connected, publish an announcement...
			client.publish(configBlock.mqttDebugTopic, "hello world");
			client.publish(configBlock.mqttDebugTopic, WiFi.localIP().toString().c_str());
		} else {
#ifdef DEBUG
			Serial.print("failed, rc=");
			Serial.print(client.state());
			Serial.println(" try again in 5 seconds");
#endif
			// Wait 5 seconds before retrying
			delay(5000);
		}
	}
}


bool mqtt_connect() {
	bool reconnected = false;
	if (!client.connected()) {
		reconnect();
		reconnected = true;
	}
	client.loop();
	return reconnected;
}


void setupProduction() {
	pinMode(SWITCH_1, INPUT_PULLUP);
	attachInterrupt(SWITCH_1, change1, CHANGE);
	pinMode(LED_1, OUTPUT);
	pinMode(SWITCH_2, INPUT_PULLUP);
	attachInterrupt(SWITCH_2, change2, CHANGE);
	pinMode(LED_2, OUTPUT);
	pinMode(SWITCH_3, INPUT_PULLUP);
	attachInterrupt(SWITCH_3, change3, CHANGE);
	pinMode(LED_3, OUTPUT);
	pinMode(SWITCH_4, INPUT_PULLUP);
	attachInterrupt(SWITCH_4, change4, CHANGE);
	pinMode(LED_4, OUTPUT);

	setup_wifi();
	client.setServer(configBlock.mqttBroker, configBlock.mqttPort);
}


void loopProduction() {
	bool reconnected = mqtt_connect();
	static uint32_t reconnectTime = 0;
	if (reconnected) {
		reconnectTime = millis();
	}

	uint32_t currentMillis = millis();


	static uint32_t lastUptimeMillis = 0;
	if (currentMillis > (lastUptimeMillis + 1000)) {
		lastUptimeMillis = currentMillis;
		uptime++;
	}

	if (pressed1 == SHORT) {
		char buf[BUF_SIZE];
		snprintf(buf, BUF_SIZE-1, "%s/1", configBlock.mqttTopic);
		client.publish(buf, "PRESS_SHORT");
		pressed1 = IDLE;
	}
	if (pressed2 == SHORT) {
		char buf[BUF_SIZE];
		snprintf(buf, BUF_SIZE-1, "%s/2", configBlock.mqttTopic);
		client.publish(buf, "PRESS_SHORT");
		pressed2 = IDLE;
	}
	if (pressed3 == SHORT) {
		char buf[BUF_SIZE];
		snprintf(buf, BUF_SIZE-1, "%s/3", configBlock.mqttTopic);
		client.publish(buf, "PRESS_SHORT");
		pressed3 = IDLE;
	}
	if (pressed4 == SHORT) {
		char buf[BUF_SIZE];
		snprintf(buf, BUF_SIZE-1, "%s/4", configBlock.mqttTopic);
		client.publish(buf, "PRESS_SHORT");
		pressed4 = IDLE;
	}
}




